@@ -19,6 +19,7 @@
 
 #include "config.h"
 
+#include <locale.h>
 #include <glib.h>
 #include <gio/gio.h>
 
@@ -105,6 +106,8 @@ main (int argc, char *argv[])
 		{ NULL }
 	};
 
+	setlocale (LC_ALL, "");
+
 	g_type_init ();
 	g_test_init (&argc, &argv, NULL);
 	g_test_bug_base ("http://bugzilla.gnome.org/show_bug.cgi?id=");
