@@ -117,6 +117,7 @@ TotemPlParserResult
 totem_pl_parser_add_iso (TotemPlParser *parser,
 			 GFile *file,
 			 GFile *base_file,
+			 TotemPlParseData *parse_data,
 			 gpointer data)
 {
 	TotemDiscMediaType type;
@@ -141,7 +142,9 @@ totem_pl_parser_add_iso (TotemPlParser *parser,
 TotemPlParserResult
 totem_pl_parser_add_cue (TotemPlParser *parser,
 			 GFile *file,
-			 GFile *base_file, gpointer data)
+			 GFile *base_file,
+			 TotemPlParseData *parse_data,
+			 gpointer data)
 {
 	char *vcduri, *path;
 
@@ -194,19 +197,25 @@ totem_pl_parser_dir_compare (GFileInfo *a, GFileInfo *b)
 }
 
 static gboolean
-totem_pl_parser_load_directory (GFile *file, GList **list)
+totem_pl_parser_load_directory (GFile *file, GList **list, gboolean *unhandled)
 {
 	GFileEnumerator *e;
 	GFileInfo *info;
+	GError *err = NULL;
 
 	*list = NULL;
+	*unhandled = FALSE;
 
 	e = g_file_enumerate_children (file,
 				       G_FILE_ATTRIBUTE_STANDARD_NAME,
 				       G_FILE_QUERY_INFO_NONE,
-				       NULL, NULL);
-	if (e == NULL)
+				       NULL, &err);
+	if (e == NULL) {
+		if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED) != FALSE)
+			*unhandled = TRUE;
+		g_error_free (err);
 		return FALSE;
+	}
 
 	while ((info = g_file_enumerator_next_file (e, NULL, NULL)) != NULL)
 		*list = g_list_prepend (*list, info);
@@ -218,20 +227,22 @@ TotemPlParserResult
 totem_pl_parser_add_directory (TotemPlParser *parser,
 			       GFile *file,
 			       GFile *base_file,
+			       TotemPlParseData *parse_data,
 			       gpointer data)
 {
 	TotemDiscMediaType type;
 	GList *list, *l;
 	char *media_uri, *uri;
+	gboolean unhandled;
 
 	uri = g_file_get_uri (file);
 	type = totem_cd_detect_type_from_dir (uri, &media_uri, NULL);
+	g_free (uri);
 
 	if (type != MEDIA_TYPE_DATA && type != MEDIA_TYPE_ERROR && media_uri != NULL) {
 		char *basename = NULL, *fname;
 
-		fname = g_filename_from_uri (uri, NULL, NULL);
-		g_free (uri);
+		fname = g_file_get_path (file);
 		if (fname != NULL) {
 			basename = g_filename_display_basename (fname);
 			g_free (fname);
@@ -242,8 +253,11 @@ totem_pl_parser_add_directory (TotemPlParser *parser,
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
-	if (totem_pl_parser_load_directory (file, &list) == FALSE)
+	if (totem_pl_parser_load_directory (file, &list, &unhandled) == FALSE) {
+		if (unhandled != FALSE)
+			return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 		return TOTEM_PL_PARSER_RESULT_ERROR;
+	}
 
 	list = g_list_sort (list, (GCompareFunc) totem_pl_parser_dir_compare);
 	l = list;
@@ -255,7 +269,7 @@ totem_pl_parser_add_directory (TotemPlParser *parser,
 
 		item = g_file_get_child (file, g_file_info_get_name (info));
 
-		ret = totem_pl_parser_parse_internal (parser, item, NULL);
+		ret = totem_pl_parser_parse_internal (parser, item, NULL, parse_data);
 		if (ret != TOTEM_PL_PARSER_RESULT_SUCCESS && ret != TOTEM_PL_PARSER_RESULT_IGNORED) {
 			char *item_uri;
 
@@ -271,7 +285,6 @@ totem_pl_parser_add_directory (TotemPlParser *parser,
 	}
 
 	g_list_free (list);
-	g_free (uri);
 
 	return TOTEM_PL_PARSER_RESULT_SUCCESS;
 }
@@ -280,6 +293,7 @@ TotemPlParserResult
 totem_pl_parser_add_block (TotemPlParser *parser,
 			   GFile *file,
 			   GFile *base_file,
+			   TotemPlParseData *parse_data,
 			   gpointer data)
 {
 	TotemDiscMediaType type;
